searchData={"items":[{"type":"module","title":"Rewrite","doc":"`Rewrite` is a tool for modifying, adding and removing files in a `Mix` project.\n\nThe package is intended for use in `Mix` tasks. `Rewrite` itself uses functions \nprovided by `Mix`. \n\nWith `Rewrite.read!/2` you can load the whole project. Then you can modify the \nproject with a number of functions provided by `Rewrite` and `Rewrite.Source` \nwithout writing any changes back to the file system. All changes are stored in \nthe source structs. Any version of a source is available in the project. To \nwrite the whole project back to the file system, the `Rewrite.write_all/2` can\nbe used.\n\nElixir source files can be modified by modifying the AST. For this `Rewrite` \nuses the `Sourceror` package to create the AST and to convert it back. The\n`Sourceror` package also provides all the utilities needed to manipulate the\nAST.\n\nSources can also receive a `Rewrite.Issue` to document problems or information \nwith the source. \n\n`Rewrite` respects the `.formatter.exs` in the project when rewriting sources. \nTo do this, the formatter can be read by `Rewrite.DotFormatter` and the \nresulting DotFormatter struct can be used in the function to update the \nsources.","ref":"Rewrite.html"},{"type":"function","title":"Rewrite.count/2","doc":"Counts the sources with the given `extname` in the `rewrite` project.","ref":"Rewrite.html#count/2"},{"type":"function","title":"Rewrite.create_source/4","doc":"Creates a new `%Source{}` without putting it to the `%Rewrite{}` project.\n\nThe `:filetypes` option of the project is used to create the source. If \noptions have been specified for the file type, the given options will be \nmerged into those options. If no `path` is given, the default file type is \ncreated.\n\nThe function does not check whether the `%Rewrite{}` project already has a \n`%Source{}` with the specified path.\n\nUse `new_source/4` if the source is to be inserted directly into the project.","ref":"Rewrite.html#create_source/4"},{"type":"function","title":"Rewrite.delete/2","doc":"Deletes the source for the given `path` from the `rewrite`. \n\nThe file system files are not removed, even if the project is written. Use \n`rm/2` or `rm!/2` to delete a file and source.\n\nIf the source is not part of the `rewrite` project the unchanged `rewrite` is\nreturned.","ref":"Rewrite.html#delete/2"},{"type":"function","title":"Examples - Rewrite.delete/2","doc":"iex> {:ok, project} = Rewrite.from_sources([\n    ...>   Source.from_string(\":a\", path: \"a.exs\"),\n    ...>   Source.from_string(\":b\", path: \"b.exs\"),\n    ...>   Source.from_string(\":a\", path: \"c.exs\")\n    ...> ])\n    iex> Rewrite.paths(project)\n    [\"a.exs\", \"b.exs\", \"c.exs\"]\n    iex> project = Rewrite.delete(project, \"a.exs\")\n    iex> Rewrite.paths(project)\n    [\"b.exs\", \"c.exs\"]\n    iex> project = Rewrite.delete(project, \"b.exs\")\n    iex> Rewrite.paths(project)\n    [\"c.exs\"]\n    iex> project = Rewrite.delete(project, \"b.exs\")\n    iex> Rewrite.paths(project)\n    [\"c.exs\"]","ref":"Rewrite.html#delete/2-examples"},{"type":"function","title":"Rewrite.dot_formatter/1","doc":"Returns the `DotFormatter` for the given `rewrite` project.\n\nWhen no formatter is set, the default formatter from \n`Rewrite.DotFormatter.default/0` is returned. A dot formatter can be set with\n`dot_formatter/2`.","ref":"Rewrite.html#dot_formatter/1"},{"type":"function","title":"Rewrite.dot_formatter/2","doc":"Sets a `dot_formatter` for the given `rewrite` project.","ref":"Rewrite.html#dot_formatter/2"},{"type":"function","title":"Rewrite.drop/2","doc":"Drops the sources with the given `paths` from the `rewrite` project.\n\nThe file system files are not removed, even if the project is written. Use \n`rm/2` or `rm!/2` to delete a file and source.\n\nIf `paths` contains paths that are not in `rewrite`, they're simply ignored.","ref":"Rewrite.html#drop/2"},{"type":"function","title":"Examples - Rewrite.drop/2","doc":"iex> {:ok, project} = Rewrite.from_sources([\n    ...>   Source.from_string(\":a\", path: \"a.exs\"),\n    ...>   Source.from_string(\":b\", path: \"b.exs\"),\n    ...>   Source.from_string(\":a\", path: \"c.exs\")\n    ...> ])\n    iex> project = Rewrite.drop(project, [\"a.exs\", \"b.exs\", \"z.exs\"])\n    iex> Rewrite.paths(project)\n    [\"c.exs\"]","ref":"Rewrite.html#drop/2-examples"},{"type":"function","title":"Rewrite.extension_for_file/2","doc":"Returns the extension of the given `file`.","ref":"Rewrite.html#extension_for_file/2"},{"type":"function","title":"Rewrite.format/2","doc":"Formats the given `rewrite` project with the given `dot_formatter`.\n\nUses the formatter from `dot_formatter/2` if no formatter ist set by \n`:dot_formatter` in the options. The other options are the same as for \n`DotFormatter.read!/2`.","ref":"Rewrite.html#format/2"},{"type":"function","title":"Rewrite.format!/2","doc":"The same as `format/2` but raises an exception in case of an error.","ref":"Rewrite.html#format!/2"},{"type":"function","title":"Rewrite.format_source/3","doc":"Formats a source in a `rewrite` project.\n\nUses the formatter from `dot_formatter/2` if no formatter ist set by \n`:dot_formatter` in the options. The other options are the same as for \n`Code.format_string!/2`.","ref":"Rewrite.html#format_source/3"},{"type":"function","title":"Rewrite.format_source!/3","doc":"The same as `format_source/3` but raises an exception in case of an error.","ref":"Rewrite.html#format_source!/3"},{"type":"function","title":"Rewrite.from_sources/2","doc":"Creates a `%Rewrite{}` from the given sources.\n\nReturns `{:ok, rewrite}` for a list of regular sources.\n\nReturns `{:error, error}` for sources with a missing path and/or duplicated\npaths.","ref":"Rewrite.html#from_sources/2"},{"type":"function","title":"Rewrite.from_sources!/2","doc":"Same as `from_sources/2`, but raises a `Rewrite.Error` exception in case of\nfailure.","ref":"Rewrite.html#from_sources!/2"},{"type":"function","title":"Rewrite.has_source?/2","doc":"Returns `true` when the `%Rewrite{}` contains a `%Source{}` with the given\n`path`.","ref":"Rewrite.html#has_source?/2"},{"type":"function","title":"Examples - Rewrite.has_source?/2","doc":"iex> {:ok, project} = Rewrite.from_sources([\n    ...>   Source.from_string(\":a\", path: \"a.exs\")\n    ...> ])\n    iex> Rewrite.has_source?(project, \"a.exs\")\n    true\n    iex> Rewrite.has_source?(project, \"b.exs\")\n    false","ref":"Rewrite.html#has_source?/2-examples"},{"type":"function","title":"Rewrite.issues?/1","doc":"Returns `true` if any source has one or more issues.","ref":"Rewrite.html#issues?/1"},{"type":"function","title":"Rewrite.map/2","doc":"Invokes `fun` for each `source` in the `rewrite` project and updates the\n`rewirte` project with the result of `fun`.\n\nReturns a `{:ok, rewrite}` if any update is successful.\n\nReturns `{:error, errors, rewrite}` where `rewrite` is updated for all sources\nthat are updated successful. The `errors` are the `errors` of `update/3`.","ref":"Rewrite.html#map/2"},{"type":"function","title":"Rewrite.map!/2","doc":"Return a `rewrite` project where each `source` is the result of invoking\n`fun` on each `source` of the given `rewrite` project.","ref":"Rewrite.html#map!/2"},{"type":"function","title":"Rewrite.move/4","doc":"Moves a source from one path to another.","ref":"Rewrite.html#move/4"},{"type":"function","title":"Rewrite.move!/4","doc":"Same as `move/4`, but raises an exception in case of failure.","ref":"Rewrite.html#move!/4"},{"type":"function","title":"Rewrite.new/1","doc":"Creates an empty project.","ref":"Rewrite.html#new/1"},{"type":"function","title":"Options - Rewrite.new/1","doc":"* `:filetypes` - a list of modules implementing the behavior \n    `Rewrite.Filetype`. This list is used to add the `filetype` to the \n    `sources` of the corresponding files. The list can contain modules \n    representing a file type or a tuple of `{module(), keyword()}`. Rewrite \n    uses the keyword list from the tuple as the options argument when a file\n    is read.\n\n    Defaults to `[Rewrite.Source, Rewrite.Source.Ex]`.\n\n  * `:dot_formatter` - a `%DotFormatter{}` that is used to format sources.\n    To get and update a dot formatter see `dot_formatter/2` and to create one\n    see `Rewrite.DotFormatter`.","ref":"Rewrite.html#new/1-options"},{"type":"function","title":"Examples - Rewrite.new/1","doc":"iex> project = Rewrite.new()\n    iex> path = \"test/fixtures/source/hello.txt\"\n    iex> project = Rewrite.read!(project, path)\n    iex> project |> Rewrite.source!(path) |> Source.get(:content)\n    \"hello\\n\"\n    iex> project |> Rewrite.source!(path) |> Source.owner()\n    Rewrite\n\n    iex> project = Rewrite.new(filetypes: [{Rewrite.Source, owner: MyApp}])\n    iex> path = \"test/fixtures/source/hello.txt\"\n    iex> project = Rewrite.read!(project, path)\n    iex> project |> Rewrite.source!(path) |> Source.owner()\n    MyApp","ref":"Rewrite.html#new/1-examples"},{"type":"function","title":"Rewrite.new!/2","doc":"Creates a `%Rewrite{}` from the given `inputs`.\n\nAccepts the same options as `new/1`.","ref":"Rewrite.html#new!/2"},{"type":"function","title":"Rewrite.new_source/4","doc":"Creates a new `%Source{}` and puts the source to the `%Rewrite{}` project.\n\nThe `:filetypes` option of the project is used to create the source. If \noptions have been specified for the file type, the given options will be \nmerged into those options.\n\nUse `create_source/4` if the source is not to be inserted directly into the \nproject.","ref":"Rewrite.html#new_source/4"},{"type":"function","title":"Rewrite.new_source!/4","doc":"Same as `new_source/4`, but raises a `Rewrite.Error` exception in case of failure.","ref":"Rewrite.html#new_source!/4"},{"type":"function","title":"Rewrite.paths/1","doc":"Returns a sorted list of all paths in the `rewrite` project.","ref":"Rewrite.html#paths/1"},{"type":"function","title":"Rewrite.put/2","doc":"Puts the given `source` to the given `rewrite` project.\n\nReturns `{:ok, rewrite}` if successful, `{:error, reason}` otherwise.","ref":"Rewrite.html#put/2"},{"type":"function","title":"Examples - Rewrite.put/2","doc":"iex> project = Rewrite.new()\n    iex> {:ok, project} = Rewrite.put(project, Source.from_string(\":a\", path: \"a.exs\"))\n    iex> map_size(project.sources)\n    1\n    iex> Rewrite.put(project, Source.from_string(\":b\"))\n    {:error, %Rewrite.Error{reason: :nopath}}\n    iex> Rewrite.put(project, Source.from_string(\":a\", path: \"a.exs\"))\n    {:error, %Rewrite.Error{reason: :overwrites, path: \"a.exs\"}}","ref":"Rewrite.html#put/2-examples"},{"type":"function","title":"Rewrite.put!/2","doc":"Same as `put/2`, but raises a `Rewrite.Error` exception in case of failure.","ref":"Rewrite.html#put!/2"},{"type":"function","title":"Rewrite.read!/3","doc":"Reads the given `input`/`inputs` and adds the source/sources to the `project`\nwhen not already readed.","ref":"Rewrite.html#read!/3"},{"type":"function","title":"Options - Rewrite.read!/3","doc":"* `:force`, default: `false` - forces the reading of sources. With\n    `force: true` updates and issues for an already existing source are \n    deleted.","ref":"Rewrite.html#read!/3-options"},{"type":"function","title":"Rewrite.rm/2","doc":"Tries to delete the `source` file in the file system and removes the `source` \nfrom the `rewrite` project.\n\nReturns `{:ok, rewrite}` if successful, or `{:error, error}` if an error\noccurs.\n\nNote the file is deleted even if in read-only mode.","ref":"Rewrite.html#rm/2"},{"type":"function","title":"Rewrite.rm!/2","doc":"Same as `source/2`, but raises a `Rewrite.Error` exception in case of failure.","ref":"Rewrite.html#rm!/2"},{"type":"function","title":"Rewrite.source/2","doc":"Returns the `%Rewrite.Source{}` for the given `path`.\n\nReturns an `:ok` tuple with the found source, if not exactly one source is\navailable an `:error` is returned.\n\nSee also `sources/2` to get a list of sources for a given `path`.","ref":"Rewrite.html#source/2"},{"type":"function","title":"Rewrite.source!/2","doc":"Same as `source/2`, but raises a `Rewrite.Error` exception in case of\nfailure.","ref":"Rewrite.html#source!/2"},{"type":"function","title":"Rewrite.sources/1","doc":"Returns all sources sorted by path.","ref":"Rewrite.html#sources/1"},{"type":"function","title":"Rewrite.update/2","doc":"Updates the given `source` in the `rewrite` project.\n\nThis function will be usually used if the `path` for the `source` has not\nchanged.\n\nReturns `{:ok, rewrite}` if successful, `{:error, error}` otherwise.","ref":"Rewrite.html#update/2"},{"type":"function","title":"Rewrite.update/3","doc":"Updates a source for the given `path` in the `rewrite` project.\n\nIf `source` a `Rewrite.Source` struct the struct is used to update the\n`rewrite` project.\n\nIf `source` is a function the source for the given `path` is passed to the\nfunction and the result is used to update the `rewrite` project.\n\nReturns `{:ok, rewrite}` if the update was successful, `{:error, error}`\notherwise.","ref":"Rewrite.html#update/3"},{"type":"function","title":"Examples - Rewrite.update/3","doc":"iex> a = Source.Ex.from_string(\":a\", path: \"a.exs\")\n    iex> b = Source.Ex.from_string(\":b\", path: \"b.exs\")\n    iex> {:ok, project} = Rewrite.from_sources([a, b])\n    iex> {:ok, project} = Rewrite.update(project, \"a.exs\", Source.Ex.from_string(\":foo\", path: \"a.exs\"))\n    iex> project |> Rewrite.source!(\"a.exs\") |> Source.get(:content)\n    \":foo\"\n    iex> {:ok, project} = Rewrite.update(project, \"a.exs\", fn s -> Source.update(s, :content, \":baz\") end)\n    iex> project |> Rewrite.source!(\"a.exs\") |> Source.get(:content)\n    \":baz\"\n    iex> {:ok, project} = Rewrite.update(project, \"a.exs\", fn s -> Source.update(s, :path, \"c.exs\") end)\n    iex> Rewrite.paths(project)\n    [\"b.exs\", \"c.exs\"]\n    iex> Rewrite.update(project, \"no.exs\", Source.from_string(\":foo\", path: \"x.exs\"))\n    {:error, %Rewrite.Error{reason: :nosource, path: \"no.exs\"}}\n    iex> Rewrite.update(project, \"c.exs\", Source.from_string(\":foo\"))\n    {:error, %Rewrite.UpdateError{reason: :nopath, source: \"c.exs\"}}\n    iex> Rewrite.update(project, \"c.exs\", fn _ -> b end)\n    {:error, %Rewrite.UpdateError{reason: :overwrites, path: \"b.exs\", source: \"c.exs\"}}","ref":"Rewrite.html#update/3-examples"},{"type":"function","title":"Rewrite.update!/2","doc":"The same as `update/2` but raises a `Rewrite.Error` exception in case\nof an error.","ref":"Rewrite.html#update!/2"},{"type":"function","title":"Rewrite.update!/3","doc":"The same as `update/3` but raises a `Rewrite.Error` exception in case\nof an error.","ref":"Rewrite.html#update!/3"},{"type":"function","title":"Rewrite.update_source/5","doc":"Updates the source for the given `path` and `key` with the given `fun`.\n\nThe function combines `update/3` and `Source.update/4` in one call.","ref":"Rewrite.html#update_source/5"},{"type":"function","title":"Examples - Rewrite.update_source/5","doc":"iex> project =\n    ...>   Rewrite.new()\n    ...>   |> Rewrite.new_source!(\"test.md\", \"foo\")\n    ...>   |> Rewrite.update_source!(\"test.md\", :content, fn content ->\n    ...>     content <> \"bar\"\n    ...>   end)\n    ...>   |> Rewrite.update_source!(\"test.md\", :content, &String.upcase/1, by: MyApp)\n    iex> source = Rewrite.source!(project, \"test.md\")\n    iex> source.content\n    \"FOOBAR\"\n    iex> source.history\n    [{:content, MyApp, \"foobar\"}, {:content, Rewrite, \"foo\"}]","ref":"Rewrite.html#update_source/5-examples"},{"type":"function","title":"Rewrite.update_source!/5","doc":"The same as `update_source/5` but raises a `Rewrite.Error` exception in case\nof an error.","ref":"Rewrite.html#update_source!/5"},{"type":"function","title":"Rewrite.updated?/1","doc":"Returns `true` if any source in the `rewrite` project returns `true` for\n`Source.updated?/1`.","ref":"Rewrite.html#updated?/1"},{"type":"function","title":"Examples - Rewrite.updated?/1","doc":"iex> {:ok, project} = Rewrite.from_sources([\n    ...>   Source.Ex.from_string(\":a\", path: \"a.exs\"),\n    ...>   Source.Ex.from_string(\":b\", path: \"b.exs\"),\n    ...>   Source.Ex.from_string(\"c\", path: \"c.txt\")\n    ...> ])\n    iex> Rewrite.updated?(project)\n    false\n    iex> project = Rewrite.update!(project, \"a.exs\", fn source ->\n    ...>   Source.update(source, :quoted, \":z\")\n    ...> end)\n    iex> Rewrite.updated?(project)\n    true","ref":"Rewrite.html#updated?/1-examples"},{"type":"function","title":"Rewrite.write/3","doc":"Writes a source to disk.\n\nThe function expects a path or a `%Source{}` as first argument.\n\nReturns `{:ok, rewrite}` if the file was written successful. See also\n`Source.write/2`.\n\nIf the given `source` is not part of the `rewrite` project then it is added.","ref":"Rewrite.html#write/3"},{"type":"function","title":"Rewrite.write!/3","doc":"The same as `write/3` but raises an exception in case of an error.","ref":"Rewrite.html#write!/3"},{"type":"function","title":"Rewrite.write_all/2","doc":"Writes all sources in the `rewrite` project to disk.\n\nThis function calls `Rewrite.Source.write/1` on all sources in the `rewrite`\nproject.\n\nReturns `{:ok, rewrite}` if all sources are written successfully.\n\nReturns `{:error, reasons, rewrite}` where `rewrite` is updated for all \nsources that are written successfully.","ref":"Rewrite.html#write_all/2"},{"type":"function","title":"Options - Rewrite.write_all/2","doc":"+ `exclude` - a list paths to exclude form writting.\n+ `force`, default: `false` - forces the writting of unchanged files.","ref":"Rewrite.html#write_all/2-options"},{"type":"type","title":"Rewrite.by/0","doc":"","ref":"Rewrite.html#t:by/0"},{"type":"type","title":"Rewrite.input/0","doc":"","ref":"Rewrite.html#t:input/0"},{"type":"type","title":"Rewrite.key/0","doc":"","ref":"Rewrite.html#t:key/0"},{"type":"type","title":"Rewrite.opts/0","doc":"","ref":"Rewrite.html#t:opts/0"},{"type":"type","title":"Rewrite.t/0","doc":"","ref":"Rewrite.html#t:t/0"},{"type":"type","title":"Rewrite.updater/0","doc":"","ref":"Rewrite.html#t:updater/0"},{"type":"type","title":"Rewrite.wildcard/0","doc":"","ref":"Rewrite.html#t:wildcard/0"},{"type":"behaviour","title":"Rewrite.DotFormatter","doc":"Provides an alternative API to the Elixir dot formatter.\n\nThe `DotFormatter` has the same functionality as the code that provides the \n`mix format` task. But `DotFormatter` provides a struct for the evaluated \nformatter config to provide a more convenient API.","ref":"Rewrite.DotFormatter.html"},{"type":"function","title":"Rewrite.DotFormatter.conflicts/2","doc":"Returns a list of conflicting files and their dot-formatter paths in the given\n`dot_formatter`.\n\nA conflicting file is a file that is referenced by more than one formatter.","ref":"Rewrite.DotFormatter.html#conflicts/2"},{"type":"function","title":"Rewrite.DotFormatter.create/2","doc":"Returns a `%DotFormatter` for the given `config`.\n\nThe `opts` has the same format as the `.formatter.exs` file.","ref":"Rewrite.DotFormatter.html#create/2"},{"type":"function","title":"Rewrite.DotFormatter.create!/2","doc":"Same as `create/2`, but raises a `Rewrite.DotFormatterError` exception\nin case of failure.","ref":"Rewrite.DotFormatter.html#create!/2"},{"type":"function","title":"Rewrite.DotFormatter.default/0","doc":"Returns an empty `%DotFormatter{}` struct with inputs set to `**`.\n\nThis is useful when no `.formatter.exs` file is present.","ref":"Rewrite.DotFormatter.html#default/0"},{"type":"function","title":"Rewrite.DotFormatter.expand/3","doc":"Returns an `:ok` tuple with a list of `{path, formatter}` tuples for the given \n`dot_formatter`.\n\nIn case of an conflict, an `:error` tuple is returned. A conflicting file is a\nfile that is referenced by more than one formatter.\n\nThe formatter is a two arity function that takes a string as input and\noptions to format the string. For more information see `formatter_for_file/3`.","ref":"Rewrite.DotFormatter.html#expand/3"},{"type":"callback","title":"Rewrite.DotFormatter.features/1","doc":"Returns which features this plugin should plug into.","ref":"Rewrite.DotFormatter.html#c:features/1"},{"type":"function","title":"Rewrite.DotFormatter.format/2","doc":"Formats the files in the current directory that are specified by the given\n`dot_formatter`.\n\nThe options are the same as for `DotFormatter.read/2`.\n\nTo format a `%Rewrite{}` project, use `Rewrite.format/2`.","ref":"Rewrite.DotFormatter.html#format/2"},{"type":"callback","title":"Rewrite.DotFormatter.format/2","doc":"Receives a string to be formatted with options and returns said string.","ref":"Rewrite.DotFormatter.html#c:format/2"},{"type":"function","title":"Rewrite.DotFormatter.format_file/3","doc":"Formats the given `file` using the given `dot_formatter` and `opts`.\n\nThe options are the same as for `Code.format_string!/2`.","ref":"Rewrite.DotFormatter.html#format_file/3"},{"type":"function","title":"Rewrite.DotFormatter.format_quoted/2","doc":"A convenience function for `format_quoted/4` to format a string.\n\nThis function reads the `%DotFormatter{}` with the given `opts` and calls \n`format_quoted/4`. The used file name defaults to `nofile.ex` and can be set \nin the `opts`.","ref":"Rewrite.DotFormatter.html#format_quoted/2"},{"type":"function","title":"Options - Rewrite.DotFormatter.format_quoted/2","doc":"* The funciton accepts the same options as `read/2`.\n\n  * `:file` - the file name to use. Defaults to `nofile.ex`.","ref":"Rewrite.DotFormatter.html#format_quoted/2-options"},{"type":"function","title":"Rewrite.DotFormatter.format_quoted/4","doc":"Converts the given `quoted` expression to a string using the given\n`dot_formatter`, `file` and `opts`.\n\nThe `file` is used to determine the formatter. If no formatter is found, an\nerror tuple is returned.\n\nReturns an :ok tuple with the formatted string on success, or an error tuple \non failure.","ref":"Rewrite.DotFormatter.html#format_quoted/4"},{"type":"function","title":"Rewrite.DotFormatter.format_quoted!/2","doc":"Same as `format_quoted/2`, but raises a `Rewrite.DotFormatterError` exception\nin case of failure.","ref":"Rewrite.DotFormatter.html#format_quoted!/2"},{"type":"function","title":"Rewrite.DotFormatter.format_quoted!/4","doc":"Same as `format_quoted/4`, but raises a `Rewrite.DotFormatterError` exception\nin case of failure.","ref":"Rewrite.DotFormatter.html#format_quoted!/4"},{"type":"function","title":"Rewrite.DotFormatter.format_string/2","doc":"A convenience function for `format_string/4` to format a string.\n\nThis function reads the `%DotFormatter{}` with the given `opts` and calls \n`format_string/4`. The used file name defaults to `nofile.ex` and can be set \nin the `opts`.","ref":"Rewrite.DotFormatter.html#format_string/2"},{"type":"function","title":"Options - Rewrite.DotFormatter.format_string/2","doc":"* The funciton accepts the same options as `read/2`.\n\n  * `:file` - the file name to use. Defaults to `nofile.ex`.","ref":"Rewrite.DotFormatter.html#format_string/2-options"},{"type":"function","title":"Rewrite.DotFormatter.format_string/4","doc":"Formats the given `string` using the specified `dot_formatter`, `file`, and\n`options`.\n\nThe `file` is used to determine the formatter. \n\nReturns an :ok tuple with the formatted string on success, or an error tuple \non failure.","ref":"Rewrite.DotFormatter.html#format_string/4"},{"type":"function","title":"Rewrite.DotFormatter.format_string!/2","doc":"Same as `format_string/2`, but raises a `Rewrite.DotFormatterError` exception\nin case of failure.","ref":"Rewrite.DotFormatter.html#format_string!/2"},{"type":"function","title":"Rewrite.DotFormatter.format_string!/4","doc":"Same as `format_string/4`, but raises a `Rewrite.DotFormatterError` exception\nin case of failure.","ref":"Rewrite.DotFormatter.html#format_string!/4"},{"type":"function","title":"Rewrite.DotFormatter.formatter_for_file/3","doc":"Returns a formatter function to be used for the given `file`.\n\nThe returned function takes a string or an Elixir AST and returns the formatted\nstring. The option `:from` takes the value `:string` or `:quoted` to determine\nwhich input type is used by the formatter, defaulting to `:string`.\n\nThe function also accepts the same options as `Code.format_string!/2`, these\nare used when the formatter is called.","ref":"Rewrite.DotFormatter.html#formatter_for_file/3"},{"type":"function","title":"Rewrite.DotFormatter.formatter_opts/1","doc":"Returns the formatter options for the given `dot_formatter`.\n\nThis functions ignores the sub-formatters of the given `dot_formatter`.","ref":"Rewrite.DotFormatter.html#formatter_opts/1"},{"type":"function","title":"Rewrite.DotFormatter.formatter_opts_for_file/2","doc":"Returns the formatter options for the given `dot_formatter` and `file`.\n\nThis fucntion searches the formatter for the given `file` and returns the \nformater options for that formatter in an `:ok` tuple.\n\nIf no formatter or multiple formatters are found, an `:error` tuple is \nreturned.","ref":"Rewrite.DotFormatter.html#formatter_opts_for_file/2"},{"type":"function","title":"Rewrite.DotFormatter.from_formatter_opts/2","doc":"Creates a `%DotFormatter{}` struct from a the given `formatter_opts`.\n\nThis function ignores the sub-formatters of the given `formatter_opts`. It is\nalso assumes that the plugins are already loaded.","ref":"Rewrite.DotFormatter.html#from_formatter_opts/2"},{"type":"function","title":"Rewrite.DotFormatter.get/2","doc":"Returns the `%DotFormatter{}` struct for the given `path` or `nil` when not\nfound.","ref":"Rewrite.DotFormatter.html#get/2"},{"type":"function","title":"Rewrite.DotFormatter.inputs/1","doc":"Returns a list of all `:inputs` from the given `dot_formatter` and any \nsub-formatters.","ref":"Rewrite.DotFormatter.html#inputs/1"},{"type":"function","title":"Rewrite.DotFormatter.map/2","doc":"Returns a `%DotFormatter{}` struct with the result of invoking `fun` on the\ngiven `dot_formatter` and any sub-formatters.","ref":"Rewrite.DotFormatter.html#map/2"},{"type":"callback","title":"Rewrite.DotFormatter.quoted_to_algebra/2","doc":"Converts a quoted expression to an algebra document using Elixir's formatter\nrules.\n\nThis function works as an replacement for `Code.quoted_to_algebra/2`.","ref":"Rewrite.DotFormatter.html#c:quoted_to_algebra/2"},{"type":"function","title":"Rewrite.DotFormatter.read/2","doc":"Reads the `.formatter.exs` file in the current directory or the given\n`%Rewrite{}` project.\n\nIf a `%Rewrite{}` project is given to the function, the formatter is searched \nin the project and the latest version from the source is used. As a fallback, \nit will search the file system for the required files.\n\nThe function returns a `%DotFormatter{}` struct with all sub-formatters.","ref":"Rewrite.DotFormatter.html#read/2"},{"type":"function","title":"Options - Rewrite.DotFormatter.read/2","doc":"* `remove_plugins` - a list of plugins to remove from the formatter.\n\n  * `replace_plugins` - a list of `{old, new}` tuples to replace plugins in\n    the formatter.\n\n  * `ignore_unknown_deps` - ingores unknown dependencies in `:import_deps` \n    when set to `true`. Defaults to `false`.","ref":"Rewrite.DotFormatter.html#read/2-options"},{"type":"function","title":"Rewrite.DotFormatter.read!/2","doc":"Same as `read/2`, but raises a `Rewrite.DotFormatterError` exception\nin case of failure.","ref":"Rewrite.DotFormatter.html#read!/2"},{"type":"function","title":"Rewrite.DotFormatter.reduce/3","doc":"Invokes `fun` for each `dot_formatter` and any sub-formatters with the\naccumulator.","ref":"Rewrite.DotFormatter.html#reduce/3"},{"type":"function","title":"Rewrite.DotFormatter.up_to_date?/2","doc":"Returns `true` if the given `dot_formatter` is up to date.\n\nThe function only checks the timestamps in the `dot_formatter` struct with the\ntimestamp of the underlying file or source in the `project`.","ref":"Rewrite.DotFormatter.html#up_to_date?/2"},{"type":"function","title":"Rewrite.DotFormatter.update/3","doc":"Updates the given `dot_formatter`.\n\nThe function checks if the `dot_formatter` is up to date. If not, the `eval`\nfunction is called.\n\nAccepts the same options as `eval/2`.","ref":"Rewrite.DotFormatter.html#update/3"},{"type":"type","title":"Rewrite.DotFormatter.formatter/0","doc":"","ref":"Rewrite.DotFormatter.html#t:formatter/0"},{"type":"type","title":"Rewrite.DotFormatter.t/0","doc":"","ref":"Rewrite.DotFormatter.html#t:t/0"},{"type":"type","title":"Rewrite.DotFormatter.timestamp/0","doc":"","ref":"Rewrite.DotFormatter.html#t:timestamp/0"},{"type":"behaviour","title":"Rewrite.Filetype","doc":"The behaviour for filetypes.\n\nAn implementation of the filetype behaviour extends a source. For an example,\nsee `Rewrite.Source.Ex`.","ref":"Rewrite.Filetype.html"},{"type":"callback","title":"Rewrite.Filetype.default_path/0","doc":"Returns the default path for the `filetype`.","ref":"Rewrite.Filetype.html#c:default_path/0"},{"type":"callback","title":"Rewrite.Filetype.extensions/0","doc":"Returns a list of file type extensions for which the module is responsible.","ref":"Rewrite.Filetype.html#c:extensions/0"},{"type":"callback","title":"Rewrite.Filetype.fetch/2","doc":"Fetches the value for a specific `key` for the given `source`.\n\nIf `source` contains the given `key` then its value is returned in the shape\nof {:ok, value}. If `source` doesn't contain key, :error is returned.","ref":"Rewrite.Filetype.html#c:fetch/2"},{"type":"callback","title":"Rewrite.Filetype.fetch/3","doc":"Fetches the value for a specific `key` in a `source` for the given `version`.\n\nIf `source` contains the given `key` then its value is returned in the shape\nof {:ok, value}. If `source` doesn't contain key, :error is returned.","ref":"Rewrite.Filetype.html#c:fetch/3"},{"type":"callback","title":"Rewrite.Filetype.from_string/1","doc":"Returns a `Rewrite.Source` with a `filetype` from the given `string`.","ref":"Rewrite.Filetype.html#c:from_string/1"},{"type":"callback","title":"Rewrite.Filetype.from_string/2","doc":"Returns a `Rewrite.Source` with a `filetype` form the given, `string` and `options`.","ref":"Rewrite.Filetype.html#c:from_string/2"},{"type":"callback","title":"Rewrite.Filetype.handle_update/3","doc":"This function is called when the content or path of the `source` is updated.\n\nReturns a `%Source{}` with an updated `filetype`.","ref":"Rewrite.Filetype.html#c:handle_update/3"},{"type":"callback","title":"Rewrite.Filetype.handle_update/4","doc":"This function is called when the `source` is updated by a `key` that is\nhandled by the current `filetype`.\n\nReturns a keyword with the keys `:content` and `:filetype` to update the\n`source`.","ref":"Rewrite.Filetype.html#c:handle_update/4"},{"type":"callback","title":"Rewrite.Filetype.read!/1","doc":"Returns a `Rewrite.Source` with a `filetype` from a file.","ref":"Rewrite.Filetype.html#c:read!/1"},{"type":"callback","title":"Rewrite.Filetype.read!/2","doc":"Returns a `Rewrite.Source` with a `filetype` from a file.","ref":"Rewrite.Filetype.html#c:read!/2"},{"type":"callback","title":"Rewrite.Filetype.undo/1","doc":"This function is called after an undo of the `source`.","ref":"Rewrite.Filetype.html#c:undo/1"},{"type":"type","title":"Rewrite.Filetype.extension/0","doc":"","ref":"Rewrite.Filetype.html#t:extension/0"},{"type":"type","title":"Rewrite.Filetype.key/0","doc":"","ref":"Rewrite.Filetype.html#t:key/0"},{"type":"type","title":"Rewrite.Filetype.opts/0","doc":"","ref":"Rewrite.Filetype.html#t:opts/0"},{"type":"type","title":"Rewrite.Filetype.t/0","doc":"","ref":"Rewrite.Filetype.html#t:t/0"},{"type":"type","title":"Rewrite.Filetype.updater/0","doc":"","ref":"Rewrite.Filetype.html#t:updater/0"},{"type":"type","title":"Rewrite.Filetype.updates/0","doc":"","ref":"Rewrite.Filetype.html#t:updates/0"},{"type":"type","title":"Rewrite.Filetype.value/0","doc":"","ref":"Rewrite.Filetype.html#t:value/0"},{"type":"module","title":"Rewrite.Source","doc":"A representation of some source in a project.\n\nThe `%Source{}` contains the `content` of the file given by `path`. The module\ncontains `update/3` to update the `path` and/or the `content`. The changes are\nrecorded in the `history` list.\n\nThe struct also holds `issues` for the source.\n\nThe different versions of `content` and `path` are available via `get/3`.\n\nA source is extensible via `filetype`, see `Rewrite.Filetype`.","ref":"Rewrite.Source.html"},{"type":"function","title":"Rewrite.Source.add_issue/2","doc":"Adds the given `issue` to the `source`.","ref":"Rewrite.Source.html#add_issue/2"},{"type":"function","title":"Rewrite.Source.add_issues/2","doc":"Adds the given `issues` to the `source`.","ref":"Rewrite.Source.html#add_issues/2"},{"type":"function","title":"Rewrite.Source.default_path/1","doc":"The default `path` for the `source`.","ref":"Rewrite.Source.html#default_path/1"},{"type":"function","title":"Rewrite.Source.diff/2","doc":"Returns iodata showing all diffs of the given `source`.\n\nSee `TextDiff.format/3` for options.","ref":"Rewrite.Source.html#diff/2"},{"type":"function","title":"Examples - Rewrite.Source.diff/2","doc":"iex> code = \"\"\"\n    ...> def foo( x ) do\n    ...>   {:x,\n    ...>     x}\n    ...> end\n    ...> \"\"\"\n    iex> formatted = code |> Code.format_string!() |> IO.iodata_to_binary()\n    iex> source = Source.Ex.from_string(code)\n    iex> source |> Source.diff() |> IO.iodata_to_binary()\n    \"\"\n    iex> source\n    ...> |> Source.update(:content, formatted)\n    ...> |> Source.diff(color: false)\n    ...> |> IO.iodata_to_binary()\n    \"\"\"\n    1   - |def foo( x ) do\n    2   - |  {:x,\n    3   - |    x}\n      1 + |def foo(x) do\n      2 + |  {:x, x}\n    4 3   |end\n    5 4   |\n    \"\"\"","ref":"Rewrite.Source.html#diff/2-examples"},{"type":"function","title":"Rewrite.Source.file_changed?/1","doc":"Returns `true` if the file has been modified since it was read.\n\nIf the key `:from` does not contain `:file` the function returns `false`.","ref":"Rewrite.Source.html#file_changed?/1"},{"type":"function","title":"Examples - Rewrite.Source.file_changed?/1","doc":"iex> File.write(\"tmp/hello.txt\", \"hello\")\n    iex> source = Source.read!(\"tmp/hello.txt\")\n    iex> Source.file_changed?(source)\n    false\n    iex> File.write(\"tmp/hello.txt\", \"Hello, world!\")\n    iex> Source.file_changed?(source)\n    true\n    iex> source = Source.update(source, :path, nil)\n    iex> Source.file_changed?(source)\n    true\n    iex> File.write(\"tmp/hello.txt\", \"hello\")\n    iex> Source.file_changed?(source)\n    false\n    iex> File.rm!(\"tmp/hello.txt\")\n    iex> Source.file_changed?(source)\n    true\n\n    iex> source = Source.from_string(\"hello\")\n    iex> Source.file_changed?(source)\n    false","ref":"Rewrite.Source.html#file_changed?/1-examples"},{"type":"function","title":"Rewrite.Source.filetype/2","doc":"Sets the `filetype` for the `source`.","ref":"Rewrite.Source.html#filetype/2"},{"type":"function","title":"Rewrite.Source.format/2","doc":"Formats the given `source`.\n\nIf the `source` was formatted the `source` gets a new `:history` entry, \notherwise the unchanged `source` is returned.","ref":"Rewrite.Source.html#format/2"},{"type":"function","title":"Options - Rewrite.Source.format/2","doc":"* `by` - an `atom` or `module` that is used as `:by` key when the `source`\n    is updated. Defaults to `Rewrite`.\n\n  * `dot_formatter` - defaults to `Rewrite.DotFormatter.default/0`.\n\n  * Accepts also the same options as `Code.format_string!/2`.","ref":"Rewrite.Source.html#format/2-options"},{"type":"function","title":"Examples - Rewrite.Source.format/2","doc":"iex> source = Source.Ex.from_string(\"\"\"\n    ...> defmodule    Foo do\n    ...>     def   foo(x),   do:    bar x\n    ...>    end\n    ...> \"\"\")\n    iex> {:ok, formatted} = Source.format(source, force_do_end_blocks: true)\n    iex> formatted.content\n    \"\"\"\n    defmodule Foo do\n      def foo(x) do\n        bar(x)\n      end\n    end\n    \"\"\"\n    iex> dot_formatter = DotFormatter.from_formatter_opts(locals_without_parens: [bar: 1])\n    iex> {:ok, formatted} = Source.format(source, \n    ...>   dot_formatter: dot_formatter, force_do_end_blocks: true\n    ...> )\n    iex> formatted.content\n    \"\"\"\n    defmodule Foo do\n      def foo(x) do\n        bar x\n      end\n    end\n    \"\"\"","ref":"Rewrite.Source.html#format/2-examples"},{"type":"function","title":"Rewrite.Source.format!/2","doc":"Same as `format/2`, but raises an exception in case of failure.","ref":"Rewrite.Source.html#format!/2"},{"type":"function","title":"Rewrite.Source.from?/2","doc":"Returns true when `from` matches to value for key `:from`.","ref":"Rewrite.Source.html#from?/2"},{"type":"function","title":"Examples - Rewrite.Source.from?/2","doc":"iex> source = Source.from_string(\"hello\")\n    iex> Source.from?(source, :file)\n    false\n    iex> Source.from?(source, :string)\n    true","ref":"Rewrite.Source.html#from?/2-examples"},{"type":"function","title":"Rewrite.Source.from_string/2","doc":"Creates a new `%Source{}` from the given `string`.","ref":"Rewrite.Source.html#from_string/2"},{"type":"function","title":"Options - Rewrite.Source.from_string/2","doc":"* `:owner` - an association to the module that owns the `source`. \n\n  * `:dot_formatter` - a fromatter for the `source`.\n\n  * `path` - the path of the `source`.","ref":"Rewrite.Source.html#from_string/2-options"},{"type":"function","title":"Examples - Rewrite.Source.from_string/2","doc":"iex> source = Source.from_string(\"hello\")\n    iex> source.content\n    \"hello\"\n    iex> source.path\n    nil\n    iex> source.owner\n    Rewrite\n\n    iex> source = Source.from_string(\"hello\", path: \"hello.md\", owner: MyApp)\n    iex> source.path\n    \"hello.md\"\n    iex> source.owner\n    MyApp","ref":"Rewrite.Source.html#from_string/2-examples"},{"type":"function","title":"Rewrite.Source.get/2","doc":"Gets the value for `:content`, `:path` in `source` or a specific `key` in\n`filetype`.\n\nRaises `Rewrite.SourceKeyError` if the `key` can't be found.","ref":"Rewrite.Source.html#get/2"},{"type":"function","title":"Rewrite.Source.get/3","doc":"Gets the value for `:content`, `:path` in `source` or a specific `key` in\n`filetype` for the given `version`.\n\nRaises `Rewrite.SourceKeyError` if the `key` can't be found.","ref":"Rewrite.Source.html#get/3"},{"type":"function","title":"Examples - Rewrite.Source.get/3","doc":"iex> bar =\n    ...>   \"\"\"\n    ...>   defmodule Bar do\n    ...>      def bar, do: :bar\n    ...>   end\n    ...>   \"\"\"\n    iex> foo =\n    ...>   \"\"\"\n    ...>   defmodule Foo do\n    ...>      def foo, do: :foo\n    ...>   end\n    ...>   \"\"\"\n    iex> source = Source.Ex.from_string(bar)\n    iex> source = Source.update(source, :content, foo)\n    iex> Source.get(source, :content) == foo\n    true\n    iex> Source.get(source, :content, 2) == foo\n    true\n    iex> Source.get(source, :content, 1) == bar\n    true\n\n    iex> source =\n    ...>   \"hello\"\n    ...>   |> Source.from_string(path: \"some/where/hello.txt\")\n    ...>   |> Source.update(:path, \"some/where/else/hello.txt\")\n    ...> Source.get(source, :path, 1)\n    \"some/where/hello.txt\"\n    iex> Source.get(source, :path, 2)\n    \"some/where/else/hello.txt\"","ref":"Rewrite.Source.html#get/3-examples"},{"type":"function","title":"Rewrite.Source.has_issues?/2","doc":"Returns `true` if the `source` has issues for the given `version`.\n\nThe `version` argument also accepts `:actual` and `:all` to check whether the\n`source` has problems for the actual version or if there are problems at all.","ref":"Rewrite.Source.html#has_issues?/2"},{"type":"function","title":"Examples - Rewrite.Source.has_issues?/2","doc":"iex> source =\n    ...>   \"a + b\"\n    ...>   |> Source.Ex.from_string(path: \"some/where/plus.exs\")\n    ...>   |> Source.add_issue(%{issue: :foo})\n    ...>   |> Source.update(:path, \"some/where/else/plus.exs\")\n    ...>   |> Source.add_issue(%{issue: :bar})\n    iex> Source.has_issues?(source)\n    true\n    iex> Source.has_issues?(source, 1)\n    true\n    iex> Source.has_issues?(source, :all)\n    true\n    iex> source = Source.update(source, :content, \"a - b\")\n    iex> Source.has_issues?(source)\n    false\n    iex> Source.has_issues?(source, 2)\n    true\n    iex> Source.has_issues?(source, :all)\n    true","ref":"Rewrite.Source.html#has_issues?/2-examples"},{"type":"function","title":"Rewrite.Source.hash/1","doc":"Calculates the current hash from the given `source`.","ref":"Rewrite.Source.html#hash/1"},{"type":"function","title":"Rewrite.Source.issues/1","doc":"Returns all issues of the given `source`.","ref":"Rewrite.Source.html#issues/1"},{"type":"function","title":"Rewrite.Source.owner/1","doc":"Returns the owner of the given `source`.","ref":"Rewrite.Source.html#owner/1"},{"type":"function","title":"Rewrite.Source.put_private/3","doc":"Assigns a private `key` and `value` to the `source`.\n\nThis is not used or accessed by Rewrite, but is intended as private storage\nfor users or libraries that wish to store additional data about a source.","ref":"Rewrite.Source.html#put_private/3"},{"type":"function","title":"Examples - Rewrite.Source.put_private/3","doc":"iex> source =\n    ...>   \"a + b\"\n    ...>   |> Source.from_string()\n    ...>   |> Source.put_private(:origin, :example)\n    iex> source.private[:origin]\n    :example","ref":"Rewrite.Source.html#put_private/3-examples"},{"type":"function","title":"Rewrite.Source.read!/2","doc":"Creates a new `%Source{}` from the given `path`.","ref":"Rewrite.Source.html#read!/2"},{"type":"function","title":"Examples - Rewrite.Source.read!/2","doc":"iex> source = Source.read!(\"test/fixtures/source/hello.txt\")\n    iex> source.content\n    \"\"\"\n    hello\n    \"\"\"","ref":"Rewrite.Source.html#read!/2-examples"},{"type":"function","title":"Rewrite.Source.rm/1","doc":"Tries to delete the file `source`.\n\nReturns `:ok` if successful, or `{:error, reason}` if an error occurs.\n\nNote the file is deleted even if in read-only mode.","ref":"Rewrite.Source.html#rm/1"},{"type":"function","title":"Rewrite.Source.rm!/1","doc":"Same as `rm/1`, but raises a `Rewrite.SourceError` exception in case of\nfailure. Otherwise `:ok`.","ref":"Rewrite.Source.html#rm!/1"},{"type":"function","title":"Rewrite.Source.touch/1","doc":"Sets the `timestamp` to the current POSIX timestamp.\n\nDoes not touch the underlying file.","ref":"Rewrite.Source.html#touch/1"},{"type":"function","title":"Rewrite.Source.touch/2","doc":"Sets the `timestamp` of the given `source` to the given `timestamp`.\n\nDoes not touch the underlying file.","ref":"Rewrite.Source.html#touch/2"},{"type":"function","title":"Rewrite.Source.undo/2","doc":"Undoes the given `number` of changes.","ref":"Rewrite.Source.html#undo/2"},{"type":"function","title":"Examples - Rewrite.Source.undo/2","doc":"iex> a = Source.from_string(\"test-a\", path: \"test/foo.txt\")\n    iex> b = Source.update(a, :content, \"test-b\")\n    iex> c = Source.update(b, :path, \"test/bar.txt\")\n    iex> d = Source.update(c, :content, \"test-d\")\n    iex> d |> Source.undo() |> Source.get(:content)\n    \"test-b\"\n    iex> d |> Source.undo(1) |> Source.get(:content)\n    \"test-b\"\n    iex> d |> Source.undo(2) |> Source.get(:path)\n    \"test/foo.txt\"\n    iex> d |> Source.undo(3) |> Source.get(:content)\n    \"test-a\"\n    iex> d |> Source.undo(9) |> Source.get(:content)\n    \"test-a\"\n    iex> d |> Source.undo(9) |> Source.updated?()\n    false\n    iex> d |> Source.undo(-9) |> Source.get(:content)\n    \"test-d\"","ref":"Rewrite.Source.html#undo/2-examples"},{"type":"function","title":"Rewrite.Source.update/4","doc":"Updates the `content` or the `path` of a `source`.\n\nThe given `value` can be of type `t:value/0` or an updater function that gets\nthe current value and returns the new value.","ref":"Rewrite.Source.html#update/4"},{"type":"function","title":"Examples - Rewrite.Source.update/4","doc":"iex> source =\n    ...>   \"foo\"\n    ...>   |> Source.from_string()\n    ...>   |> Source.update(:path, \"test/fixtures/new.exs\", by: Example)\n    ...>   |> Source.update(:content, \"bar\")\n    iex> source.history\n    [{:content, Rewrite, \"foo\"}, {:path, Example, nil}]\n    iex> source.content\n    \"bar\"\n\n    iex> source =\n    ...>   \"foo\"\n    ...>   |> Source.from_string()\n    ...>   |> Source.update(:content, fn content -> content <> \"bar\" end)\n    iex> source.content\n    \"foobar\"\n\nWith a `Rewrite.Source.Ex`. Note that the AST is generated by `Sourceror`.\n\n    iex> source =\n    ...>   \":a\"\n    ...>   |> Source.Ex.from_string()\n    ...>   |> Source.update(:quoted, fn quoted ->\n    ...>     {:__block__, meta, [atom]} = quoted \n    ...>     {:__block__, meta, [{:ok, atom}]}\n    ...>   end)\n    iex> source.content\n    \"{:ok, :a}\\n\"\n\nIf the new value is equal to the current value, no history will be added.\n\n    iex> source =\n    ...>   \"42\"\n    ...>   |> Source.from_string()\n    ...>   |> Source.update(:content, \"21\", by: Example)\n    ...>   |> Source.update(:content, \"21\", by: Example)\n    iex> source.history\n    [{:content, Example, \"42\"}]","ref":"Rewrite.Source.html#update/4-examples"},{"type":"function","title":"Rewrite.Source.updated?/2","doc":"Returns `true` if the source was updated.\n\nThe optional argument `kind` specifies whether only `:code` changes or `:path`\nchanges are considered. Defaults to `:any`.","ref":"Rewrite.Source.html#updated?/2"},{"type":"function","title":"Examples - Rewrite.Source.updated?/2","doc":"iex> source = Source.from_string(\"foo\")\n    iex> Source.updated?(source)\n    false\n    iex> source = Source.update(source, :content, \"bar\")\n    iex> Source.updated?(source)\n    true\n    iex> Source.updated?(source, :path)\n    false\n    iex> Source.updated?(source, :content)\n    true","ref":"Rewrite.Source.html#updated?/2-examples"},{"type":"function","title":"Rewrite.Source.version/1","doc":"Returns the `version` of the given `source`. The value `1` indicates that the\nsource has no changes.","ref":"Rewrite.Source.html#version/1"},{"type":"function","title":"Rewrite.Source.write/2","doc":"Writes the source to disk.\n\nReturns `{:ok, source}` when the file was written successfully. The returned\n`source` does not include any previous changes or issues.\n\nIf there's an error, this function returns `{:error, error}` where `error`\nis a `Rewrite.SourceError`. You can raise it manually with `raise/1`.\n\nReturns `{:error, error}` with `reason: :nopath` if the current `path` is nil.\n\nReturns `{:error, error}` with `reason: :changed` if the file was changed\nsince reading. See also `file_changed?/1`. The option `force: true` forces\noverwriting a changed file.\n\nIf the source `:path` was updated then the old file will be deleted.\n\nMissing directories are created.","ref":"Rewrite.Source.html#write/2"},{"type":"function","title":"Options - Rewrite.Source.write/2","doc":"* `:force`, default: `false` - forces the saving to overwrite changed files.\n\n  * `:rm`, default: `true` - prevents file deletion when set to `false`.","ref":"Rewrite.Source.html#write/2-options"},{"type":"function","title":"Examples - Rewrite.Source.write/2","doc":"iex> \":test\" |> Source.from_string() |> Source.write()\n    {:error, %SourceError{reason: :nopath, path: nil, action: :write}}\n\n    iex> path = \"tmp/foo.txt\"\n    iex> File.write(path, \"foo\")\n    iex> source = path |> Source.read!() |> Source.update(:content, \"bar\")\n    iex> Source.updated?(source)\n    true\n    iex> {:ok, source} = Source.write(source)\n    iex> File.read(path)\n    {:ok, \"bar\\n\"}\n    iex> Source.updated?(source)\n    false\n\n    iex> source = Source.from_string(\"bar\")\n    iex> Source.write(source)\n    {:error, %SourceError{reason: :nopath, path: nil, action: :write}}\n    iex> source |> Source.update(:path, \"tmp/bar.txt\") |> Source.write()\n    iex> File.read(\"tmp/bar.txt\")\n    {:ok, \"bar\\n\"}\n\n    iex> path = \"tmp/ping.txt\"\n    iex> File.write(path, \"ping\")\n    iex> source = Source.read!(path)\n    iex> new_path = \"tmp/pong.ex\"\n    iex> source = Source.update(source, :path, new_path)\n    iex> Source.write(source)\n    iex> File.exists?(path)\n    false\n    iex> File.read(new_path)\n    {:ok, \"ping\\n\"}\n\n    iex> path = \"tmp/ping.txt\"\n    iex> File.write(path, \"ping\")\n    iex> source = Source.read!(path)\n    iex> new_path = \"tmp/pong.ex\"\n    iex> source = Source.update(source, :path, new_path)\n    iex> Source.write(source, rm: false)\n    iex> File.exists?(path)\n    true\n\n    iex> path = \"tmp/ping.txt\"\n    iex> File.write(path, \"ping\")\n    iex> source = path |> Source.read!() |> Source.update(:content, \"peng\")\n    iex> File.write(path, \"pong\")\n    iex> Source.write(source)\n    {:error, %SourceError{reason: :changed, path: \"tmp/ping.txt\", action: :write}}\n    iex> {:ok, _source} = Source.write(source, force: true)","ref":"Rewrite.Source.html#write/2-examples"},{"type":"function","title":"Rewrite.Source.write!/1","doc":"Same as `write/1`, but raises a `Rewrite.SourceError` exception in case of\nfailure.","ref":"Rewrite.Source.html#write!/1"},{"type":"type","title":"Rewrite.Source.by/0","doc":"","ref":"Rewrite.Source.html#t:by/0"},{"type":"type","title":"Rewrite.Source.content/0","doc":"","ref":"Rewrite.Source.html#t:content/0"},{"type":"type","title":"Rewrite.Source.extension/0","doc":"","ref":"Rewrite.Source.html#t:extension/0"},{"type":"type","title":"Rewrite.Source.filetype/0","doc":"","ref":"Rewrite.Source.html#t:filetype/0"},{"type":"type","title":"Rewrite.Source.from/0","doc":"","ref":"Rewrite.Source.html#t:from/0"},{"type":"type","title":"Rewrite.Source.issue/0","doc":"","ref":"Rewrite.Source.html#t:issue/0"},{"type":"type","title":"Rewrite.Source.key/0","doc":"","ref":"Rewrite.Source.html#t:key/0"},{"type":"type","title":"Rewrite.Source.kind/0","doc":"","ref":"Rewrite.Source.html#t:kind/0"},{"type":"type","title":"Rewrite.Source.opts/0","doc":"","ref":"Rewrite.Source.html#t:opts/0"},{"type":"type","title":"Rewrite.Source.owner/0","doc":"","ref":"Rewrite.Source.html#t:owner/0"},{"type":"type","title":"Rewrite.Source.t/0","doc":"The `struct` representing a source.","ref":"Rewrite.Source.html#t:t/0"},{"type":"type","title":"Fields - Rewrite.Source.t/0","doc":"* `content` - of the `source`.\n\n  * `filetype` - a `struct` implementing the behaviour `Rewrite.Filetype`.\n    The `filetype` is nil when no additional implementation for the `filetype`\n    is available.\n\n  * `from` - contains `:file` or `:string` depending on whether the `source`\n    is created from a file or a string.\n\n  * `hash` - of the `source`. The `hash` is built from the `content` and \n    `path`.\n\n  * `history` - of the `source`.\n\n  * `issues` - of the `source`.\n\n  * `owner` - of the `source`.\n\n  * `path` - of the `source`. Can be `nil` if the `source` was created by a \n    `string`.\n\n  * `private` - a field for user defined data.\n\n  * `timestamp` - is set to the timestamp of the last modification of the file \n    on disk at the time it was read.\n\n    If the `source` was created by a `string`, the timestamp is the creation \n    time.\n\n    The timestamp will be updated when the `source` is updated.","ref":"Rewrite.Source.html#t:t/0-fields"},{"type":"type","title":"Rewrite.Source.timestamp/0","doc":"A `timestamp` as `integer` seconds since epoch.","ref":"Rewrite.Source.html#t:timestamp/0"},{"type":"type","title":"Rewrite.Source.updater/0","doc":"","ref":"Rewrite.Source.html#t:updater/0"},{"type":"type","title":"Rewrite.Source.value/0","doc":"","ref":"Rewrite.Source.html#t:value/0"},{"type":"type","title":"Rewrite.Source.version/0","doc":"The `version` of a `%Source{}`. The version `1` indicates that the source has\nno changes.","ref":"Rewrite.Source.html#t:version/0"},{"type":"module","title":"Rewrite.Source.Ex","doc":"An implementation of `Rewrite.Filetype` to handle Elixir source files.\n\nThe module uses the [`sourceror`](https://github.com/doorgan/sourceror) package\nto provide an [extended AST](https://hexdocs.pm/sourceror/readme.html#sourceror-s-ast)\nrepresentation of an Elixir file.\n\n`Ex` extends the `source` by the key `:quoted`.","ref":"Rewrite.Source.Ex.html"},{"type":"module","title":"Updating and resyncing `:quoted` - Rewrite.Source.Ex","doc":"When `:quoted` becomes updated, content becomes formatted to the Elixir source\ncode. To keep the code in `:content` in sync with the AST in `:quoted`, the\nnew code is parsed to a new `:quoted`. That means that\n`Source.update(source, :quoted, quoted)` also updates the AST.\n\nThe resyncing of `:quoted` can be suppressed with the option \n`resync_quoted: false`.","ref":"Rewrite.Source.Ex.html#module-updating-and-resyncing-quoted"},{"type":"module","title":"Examples - Rewrite.Source.Ex","doc":"iex> source = Source.Ex.from_string(\"Enum.reverse(list)\")\n    iex> Source.get(source, :quoted)\n    {{:., [trailing_comments: [], line: 1, column: 5],\n      [\n        {:__aliases__,\n         [\n           trailing_comments: [],\n           leading_comments: [],\n           last: [line: 1, column: 1],\n           line: 1,\n           column: 1\n         ], [:Enum]},\n        :reverse\n      ]},\n     [\n       trailing_comments: [],\n       leading_comments: [],\n       closing: [line: 1, column: 18],\n       line: 1,\n       column: 6\n     ], [{:list, [trailing_comments: [], leading_comments: [], line: 1, column: 14], nil}]}\n    iex> quoted = Code.string_to_quoted!(\"\"\"\n    ...> defmodule MyApp.New do\n    ...>   def      foo do\n    ...>   :foo\n    ...> end\n    ...> end\n    ...> \"\"\")\n    iex> source = Source.update(source, :quoted, quoted)\n    iex> Source.updated?(source)\n    true\n    iex> Source.get(source, :content)\n    \"\"\"\n    defmodule MyApp.New do\n      def foo do\n        :foo\n      end\n    end\n    \"\"\"\n    iex> Source.get(source, :quoted) == quoted\n    false\n\nWithout resyncing `:quoted`:\n\n    iex> project = Rewrite.new(filetypes: [{Source.Ex, resync_quoted: false}])\n    iex> path = \"test/fixtures/source/simple.ex\"\n    iex> project = Rewrite.read!(project, path)\n    iex> source = Rewrite.source!(project, path)\n    iex> quoted = Code.string_to_quoted!(\"\"\"\n    ...> defmodule MyApp.New do\n    ...>   def      foo do\n    ...>   :foo\n    ...> end\n    ...> end\n    ...> \"\"\")\n    iex> source = Source.update(source, :quoted, quoted)\n    iex> Source.get(source, :quoted) == quoted\n    true","ref":"Rewrite.Source.Ex.html#module-examples"},{"type":"function","title":"Rewrite.Source.Ex.format/2","doc":"","ref":"Rewrite.Source.Ex.html#format/2"},{"type":"function","title":"Rewrite.Source.Ex.from_string/2","doc":"Returns a `%Rewrite.Source{}` with an added `:filetype`.","ref":"Rewrite.Source.Ex.html#from_string/2"},{"type":"function","title":"Rewrite.Source.Ex.modules/1","doc":"Returns the current modules for the given `source`.","ref":"Rewrite.Source.Ex.html#modules/1"},{"type":"function","title":"Rewrite.Source.Ex.modules/2","doc":"Returns the modules of a `source` for the given `version`.","ref":"Rewrite.Source.Ex.html#modules/2"},{"type":"function","title":"Examples - Rewrite.Source.Ex.modules/2","doc":"iex> bar =\n    ...>   \"\"\"\n    ...>   defmodule Bar do\n    ...>      def bar, do: :bar\n    ...>   end\n    ...>   \"\"\"\n    iex> foo =\n    ...>   \"\"\"\n    ...>   defmodule Baz.Foo do\n    ...>      def foo, do: :foo\n    ...>   end\n    ...>   \"\"\"\n    iex> source = Source.Ex.from_string(bar)\n    iex> source = Source.update(source, :content, bar <> foo)\n    iex> Source.Ex.modules(source)\n    [Baz.Foo, Bar]\n    iex> Source.Ex.modules(source, 2)\n    [Baz.Foo, Bar]\n    iex> Source.Ex.modules(source, 1)\n    [Bar]","ref":"Rewrite.Source.Ex.html#modules/2-examples"},{"type":"function","title":"Rewrite.Source.Ex.put_formatter_opts/2","doc":"","ref":"Rewrite.Source.Ex.html#put_formatter_opts/2"},{"type":"function","title":"Rewrite.Source.Ex.read!/2","doc":"Returns a `%Rewrite.Source{}` with an added `:filetype`.\n\nThe `content` reads from the file under the given `path`.","ref":"Rewrite.Source.Ex.html#read!/2"},{"type":"function","title":"Options - Rewrite.Source.Ex.read!/2","doc":"* `:resync_quoted`, default: `true` - forcing the re-parsing when the source\n    field `quoted` is updated.","ref":"Rewrite.Source.Ex.html#read!/2-options"},{"type":"type","title":"Rewrite.Source.Ex.t/0","doc":"","ref":"Rewrite.Source.Ex.html#t:t/0"},{"type":"behaviour","title":"Rewrite.Hook","doc":"A `behaviour` for hooking into the `Rewrite` processes.\n\nThe callback `c:handle/2` is called by `Rewrite` with an `t:action/0` and the \ncurrent `%Rewrite{}`. The return value is either `:ok` or `{:ok, rewrite}`.\n\n> #### Warning {: .warning}\n> If the `%Rewrite{}` project is updated inside the hook, the hook will be \n> called again.","ref":"Rewrite.Hook.html"},{"type":"behaviour","title":"Actions - Rewrite.Hook","doc":"* `:new` - invoked when a new `%Rewrite{}` is created.\n\n  * `{:added, paths}` - invoked when new sources were added. `paths` is the a \n    list of `t:Path.t()`.\n\n  * `{:updated, path}` - invoked when a source was updated. `path` contains \n    the path of the updated source. Also called when a source was succesfull \n    formatted.","ref":"Rewrite.Hook.html#module-actions"},{"type":"callback","title":"Rewrite.Hook.handle/2","doc":"","ref":"Rewrite.Hook.html#c:handle/2"},{"type":"type","title":"Rewrite.Hook.action/0","doc":"","ref":"Rewrite.Hook.html#t:action/0"},{"type":"module","title":"Rewrite.Hook.DotFormatterUpdater","doc":"A hook that updates the dot-formatter for a `%Rewrite{}` project on changes.","ref":"Rewrite.Hook.DotFormatterUpdater.html"},{"type":"exception","title":"Rewrite.DotFormatterError","doc":"An exception raised when an error is encountered while working with \ndot_formatters.","ref":"Rewrite.DotFormatterError.html"},{"type":"function","title":"Rewrite.DotFormatterError.message/1","doc":"","ref":"Rewrite.DotFormatterError.html#message/1"},{"type":"type","title":"Rewrite.DotFormatterError.exits/0","doc":"","ref":"Rewrite.DotFormatterError.html#t:exits/0"},{"type":"type","title":"Rewrite.DotFormatterError.reason/0","doc":"","ref":"Rewrite.DotFormatterError.html#t:reason/0"},{"type":"type","title":"Rewrite.DotFormatterError.t/0","doc":"","ref":"Rewrite.DotFormatterError.html#t:t/0"},{"type":"exception","title":"Rewrite.Error","doc":"An exception for when a function cannot handle a source.","ref":"Rewrite.Error.html"},{"type":"type","title":"Rewrite.Error.reason/0","doc":"","ref":"Rewrite.Error.html#t:reason/0"},{"type":"type","title":"Rewrite.Error.t/0","doc":"","ref":"Rewrite.Error.html#t:t/0"},{"type":"exception","title":"Rewrite.SourceError","doc":"An exception for when a function can not handle a source.","ref":"Rewrite.SourceError.html"},{"type":"function","title":"Rewrite.SourceError.format/1","doc":"","ref":"Rewrite.SourceError.html#format/1"},{"type":"type","title":"Rewrite.SourceError.action/0","doc":"","ref":"Rewrite.SourceError.html#t:action/0"},{"type":"type","title":"Rewrite.SourceError.path/0","doc":"","ref":"Rewrite.SourceError.html#t:path/0"},{"type":"type","title":"Rewrite.SourceError.reason/0","doc":"","ref":"Rewrite.SourceError.html#t:reason/0"},{"type":"type","title":"Rewrite.SourceError.t/0","doc":"","ref":"Rewrite.SourceError.html#t:t/0"},{"type":"exception","title":"Rewrite.SourceKeyError","doc":"An exception for when a key can't be found in a source.","ref":"Rewrite.SourceKeyError.html"},{"type":"exception","title":"Rewrite.UpdateError","doc":"An exception for when a function can not handle a source.","ref":"Rewrite.UpdateError.html"},{"type":"type","title":"Rewrite.UpdateError.reason/0","doc":"","ref":"Rewrite.UpdateError.html#t:reason/0"},{"type":"type","title":"Rewrite.UpdateError.t/0","doc":"","ref":"Rewrite.UpdateError.html#t:t/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}